<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Illuminate\Http\Request;

class FinanceiroController extends Controller
{
    public function index(Request $request)
{
    // Obtém o usuário autenticado
    $usuarioAutenticado = Auth::user();

    // Mantém o CNPJ do cliente autenticado para busca de conveniados
    $cnpj = $usuarioAutenticado ? $usuarioAutenticado->cnpj : null;

    if (!$cnpj) {
        return redirect()->route('auth.login')->with('error', 'Usuário não autenticado ou CPF/CNPJ não disponível');
    }

    // Busca o código do convênio do cliente logado
    $codigoConvenio = DB::connection('oracle')->table('TAB_CLIENTE')
        ->where('NUM_CGC', $cnpj)
        ->value('COD_CONVENIO');

    // Inicializa o array de CNPJs conveniados
    $cnpjconveniados = [];
    if ($codigoConvenio) {
        $clientesConveniados = DB::connection('oracle')->table('TAB_CLIENTE')
            ->where('COD_CONVENIO', $codigoConvenio)
            ->select(DB::raw("NVL(NUM_CGC, '') as NUM_CGC"))
            ->get();

        $clientesConveniadosArray = $clientesConveniados->toArray();

        if (!empty($clientesConveniadosArray)) {
            $cnpjconveniados = array_map(function ($cliente) {
                return $cliente->num_cgc;
            }, $clientesConveniadosArray);
        }
    }

    // Inicializa boletos e vencidos
    $boletos = collect();
    $boletosVencidos = false;

    // Verifica se foi passado um CNPJ específico (associado)
    if ($request->has('associado')) {
        $associado = $request->input('associado');

        // Busca o código do cliente associado
        $codigoCliente = DB::connection('oracle')->table('TAB_CLIENTE')
            ->where('NUM_CGC', $associado)
            ->value('COD_CLIENTE');

        if ($codigoCliente) {
            // Substituindo a consulta SQL pela Query Builder
            $boletos = DB::connection('oracle')->table('TAB_FLUXO_BOLETO')
            ->join('TAB_FLUXO', 'TAB_FLUXO.NUM_BORDERO', '=', 'TAB_FLUXO_BOLETO.NUM_BORDERO')
            ->join('TAB_FLUXO_REFERENCIA', 'TAB_FLUXO.COD_CHAVE_CTB', '=', 'TAB_FLUXO_REFERENCIA.COD_CHAVE_REF')
            ->join('TAB_LOJA', 'TAB_FLUXO.COD_LOJA', '=', 'TAB_LOJA.COD_LOJA')
            ->leftJoin('TAB_CLIENTE_NOTA', function ($join) {
                $join->on('TAB_FLUXO_REFERENCIA.COD_PARCEIRO', '=', 'TAB_CLIENTE_NOTA.COD_CLIENTE')
                     ->on('TAB_FLUXO_REFERENCIA.NUM_NF', '=', 'TAB_CLIENTE_NOTA.NUM_NF_CLI')
                     ->on('TAB_FLUXO_REFERENCIA.NUM_SERIE_NF', '=', 'TAB_CLIENTE_NOTA.NUM_SERIE_NF')
                     ->on('TAB_FLUXO_REFERENCIA.COD_LOJA', '=', 'TAB_CLIENTE_NOTA.COD_LOJA');
            })
            ->select([
                'TAB_LOJA.DES_LOJA as beneficiario',
                'TAB_CLIENTE_NOTA.NUM_NF_CLI as numero_nota',
                'TAB_CLIENTE_NOTA.NUM_SERIE_NF as num_serie_nf',
                'TAB_FLUXO_BOLETO.DTA_EMISSAO as data_emissao',
                'TAB_FLUXO_BOLETO.NUM_REGISTRO as num_registro',
                'TAB_FLUXO_BOLETO.DTA_VENCIMENTO as data_vencimento',
                'TAB_FLUXO_BOLETO.VAL_BOLETO as valor',
                'TAB_FLUXO.FLG_QUITADO as status',
                'TAB_FLUXO_BOLETO.LINHA_DIGITAVEL as linha_digitavel',
                DB::raw('ROW_NUMBER() OVER (PARTITION BY TAB_FLUXO_BOLETO.LINHA_DIGITAVEL ORDER BY TAB_FLUXO_BOLETO.DTA_VENCIMENTO DESC) AS rn')
            ])
            ->where('TAB_CLIENTE_NOTA.COD_CLIENTE', $codigoCliente) // Use o código do cliente aqui
            ->where('TAB_FLUXO_BOLETO.DTA_EMISSAO', '>=', DB::raw("ADD_MONTHS(SYSDATE, -12)"))
            ->whereNotNull('TAB_FLUXO_BOLETO.PDF_BOLETO')
            ->groupBy( // Agrupa pelos campos relevantes para evitar duplicatas
                'TAB_LOJA.DES_LOJA',
                'TAB_CLIENTE_NOTA.NUM_NF_CLI',
                'TAB_CLIENTE_NOTA.NUM_SERIE_NF',
                'TAB_FLUXO_BOLETO.DTA_EMISSAO',
                'TAB_FLUXO_BOLETO.NUM_REGISTRO',
                'TAB_FLUXO_BOLETO.DTA_VENCIMENTO',
                'TAB_FLUXO_BOLETO.VAL_BOLETO',
                'TAB_FLUXO.FLG_QUITADO',
                'TAB_FLUXO_BOLETO.LINHA_DIGITAVEL'
            )
            ->orderBy('TAB_FLUXO_BOLETO.DTA_VENCIMENTO', 'DESC')
            ->get()
            ->filter(function ($item) {
                return $item->rn == 1; // Filtra para trazer apenas a linha com ROW_NUMBER = 1
            });

            $boletos = $boletos->unique('numero_nota');

            // Verifica se existem boletos vencidos
            $boletosVencidos = $boletos->some(function ($boleto) {
                $dataVencimento = Carbon::parse($boleto->data_vencimento)->startOfDay();
                return $boleto->status === 'N' && $dataVencimento->isPast();
            });
        }
    }

    return view('dashboard', [
        'boletos' => $boletos,
        'boletosVencidos' => $boletosVencidos,
        'cnpjconveniados' => $cnpjconveniados, // Mantém para exibição no select
    ]);
}

}
